let keyCodeMap: {[key: number]: string} = {
    40: "ArrowDown",
    37: "ArrowLeft",
    39: "ArrowRight",
    38: "ArrowUp",
    65: "KeyA",
    66: "KeyB",
    67: "KeyC",
    68: "KeyD",
    69: "KeyE",
    70: "KeyF",
    71: "KeyG",
    72: "KeyH",
    73: "KeyI",
    74: "KeyJ",
    75: "KeyK",
    76: "KeyL",
    77: "KeyM",
    78: "KeyN",
    79: "KeyO",
    80: "KeyP",
    81: "KeyQ",
    82: "KeyR",
    83: "KeyS",
    84: "KeyT",
    85: "KeyU",
    86: "KeyV",
    87: "KeyW",
    88: "KeyX",
    89: "KeyY",
    90: "KeyZ"
}

function getKeyboardEventCode(e: KeyboardEvent): string | null {
    if(e.code) return e.code;
    if(keyCodeMap.hasOwnProperty(e.keyCode)) {
        return keyCodeMap[e.keyCode];
    }
    return null;
}

class BindingLayer extends PH.Layer {

    game: Game;
    skill: Skill;
    callback: (binding: string) => void;

    midx = 320;
    midy = 180;
    width = 208;
    height = 64;
    left = this.midx - this.width / 2;
    top = this.midy - this.height / 2;

    reservedKeys = ["ArrowUp", "ArrowDown", "ArrowLeft", "ArrowRight"];
    chosen: string | null = null;
    errMsg: string | null = null;

    constructor(game: Game, skill: Skill, callback: (binding: string) => void) {
        super();
        this.game = game;
        this.skill = skill;
        this.callback = callback;
    }

    draw() {
        let ctx = this.game.ctx;
        let mainFont = this.game.mainFont;

        ctx.fillStyle = "#000000"
        ctx.globalAlpha = 0.5;
        ctx.fillRect(0, 0, 640, 360);
        ctx.globalAlpha = 1;

        this.game.spriteBoxFrame.draw(ctx, this.left, this.top, this.width, this.height);
        mainFont.drawCenteredText(ctx, "Press a key to bind to", this.midx, this.top + 14);
        mainFont.drawCenteredText(ctx, this.skill.name, this.midx, this.top + 28);
        if(this.chosen !== null) {
            mainFont.drawCenteredText(ctx, this.chosen, this.midx, this.top + 50);
        }
        else if(this.errMsg !== null) {
            mainFont.drawCenteredText(ctx, this.errMsg, this.midx, this.top + 50);
        }
    }

    handleClick() { return false; }
    handleDoubleClick() { return false; }
    handleMouseDown() { return false; }
    handleMouseUp() { return false; }
    handleKeyDown(e: KeyboardEvent) {
        let code = getKeyboardEventCode(e);
        if(code === null) return true;
        if(this.reservedKeys.indexOf(code) < 0) {
            this.chosen = code;
            setTimeout(() => {
                this.callback(code!);
            }, 750);
        }
        else {
            this.errMsg = `${code} is reserved for movement.`;
        }
        return false;
    }
    handleKeyUp() { return false; }
}