
abstract class Collider {
    pos: [number, number];
    lastPos: [number, number];
    rad: number;
    keep = true;
    ghost = false;

    constructor(pos: [number, number], rad: number) {
        this.pos = pos;
        this.lastPos = pos;
        this.rad = rad;
    }

    moveTo(pos: [number, number], teleport?: boolean) {
        this.lastPos = this.pos;
        this.pos = pos;
        if(teleport) {
            this.lastPos = pos;
        }
    }
}


function checkCollision(objA: Collider, objB: Collider): boolean {
    // Return true if there is a collision.
    // (If there were fast objects, we may have to account for lastPos,
    // and do some algebra. I haven't experienced this problem at
    // the speeds I'm dealing with, and it would be a huge side-track
    // to do that.)
    let pa = objA.pos;
    let ra = objA.rad;
    let pb = objB.pos;
    let rb = objB.rad;
    let dx = pa[0] - pb[0];
    let dy = pa[1] - pb[1];
    let dsq = dx * dx + dy * dy;
    let rsumsq = (ra + rb) * (ra + rb);
    return (dsq <= rsumsq);
}
