function quickSplit(msg: string, font: PH.CanvasFont, width: number) {
    let splitMsg = msg.split('\n');
    let lines = [];
    for (let k = 0; k < splitMsg.length; k++) {
        let s = splitMsg[k];
        let wrapped = font.wordWrap(s, width - 16);
        for (let ws of wrapped) {
            lines.push(ws);
        }
        if (k < splitMsg.length - 1) lines.push('');
    }
    return lines;
}

class MsgBoxLayer extends PH.Layer {

    game: Game;
    msgLines: string[] = [];
    callback: (() => void) | null = null;
    active = false;
    canClose = false;

    midx = 320;
    midy = 180;
    width = 360;
    left = this.midx - this.width / 2;
    height = 0;
    top = 0;

    constructor(game: Game) {
        super();
        this.game = game;
    }

    display(msg: string, callback: () => void) {
        this.msgLines = quickSplit(msg, this.game.mainFont, this.width);
        this.height = (this.msgLines.length + 2) * 14 + 16;
        this.top = this.midy - this.height / 2;
        this.callback = callback;
        this.active = true;
        this.canClose = false;

        setTimeout(() => {
            this.canClose = true;
            this.msgLines.push('');
            this.msgLines.push("Click to continue.");
        }, 500);
    }

    draw() {
        if (this.active) {
            let ctx = this.game.ctx;
            let mainFont = this.game.mainFont;

            ctx.fillStyle = "#000000"
            ctx.globalAlpha = 0.5;
            ctx.fillRect(0, 0, 640, 360);
            ctx.globalAlpha = 1;

            this.game.spriteBoxFrame.draw(ctx, this.left, this.top, this.width, this.height);

            for (let k = 0; k < this.msgLines.length; k++) {
                let y = this.top + 15 + 14 * k;
                mainFont.drawCenteredText(ctx, this.msgLines[k], this.midx, y);
            }
        }
    }

    handleClick() {
        if (!this.active) return true;
        if (this.canClose) {
            this.msgLines = [];
            this.active = false;
            if (this.callback !== null) {
                this.callback();
            }
            this.callback = null;
        }
        return false;
    }
    handleDoubleClick() { return !this.active; }
    handleMouseDown() { return !this.active; }
    handleMouseUp() { return !this.active; }
    handleKeyDown() { return !this.active; }
    handleKeyUp() { return !this.active; }
}