// Enemy pattern generators

namespace Patterns {

    export async function basicEnemy(sl: ShmupLayer, x: number, y: number) {
        sl.enemies.push(new BasicEnemy(sl, [x, y]));
    }

    export async function ghostEnemy(sl: ShmupLayer, x: number, y: number) {
        sl.enemies.push(new GhostEnemy(sl, [x, y]));
    }

    export async function kamikazeEnemy(sl: ShmupLayer, x: number, y: number, amp: number) {
        sl.enemies.push(new KamikazeEnemy(sl, [x, y], amp));
    }

    export async function trackerEnemy(sl: ShmupLayer, x: number, y: number, numShots: number) {
        sl.enemies.push(new TrackerEnemy(sl, [x, y], numShots));
    }

    export async function mountainEnemy(sl: ShmupLayer, x: number, y: number) {
        sl.enemies.push(new MountainEnemy(sl, [x, y]));
    }

    export async function bossEnemy(sl: ShmupLayer) {
        sl.enemies.push(new BossEnemy(sl));
    }

    export async function sineY(sl: ShmupLayer, duration: number,
        waveCount: number, enemyCount: number,
        startY: number, endY: number,
        f: (sl: ShmupLayer, x: number, y: number) => void) {
        let startTime = sl.playTime;
        let ts = duration / (enemyCount - 1);
        let mul = 2 * Math.PI * waveCount / (enemyCount - 1);
        let mid = (startY + endY) / 2;
        let amp = (startY - endY) / 2;
        for (let k = 0; k < enemyCount; k++) {
            // create some new projectiles
            let y = Math.cos(mul * k) * amp + mid;
            f(sl, FLY_START, y);
            await sl.waitUntilTime(startTime + ts * (k + 1));
        }
    }

    export async function lineY(sl: ShmupLayer, duration: number,
        enemyCount: number,
        startY: number, endY: number,
        f: (sl: ShmupLayer, x: number, y: number) => void) {
        let startTime = sl.playTime;
        let ts = duration / (enemyCount - 1);
        for (let k = 0; k < enemyCount; k++) {
            // create some new projectiles
            let y = startY + (endY - startY) * k / (enemyCount - 1);
            f(sl, FLY_START, y);
            await sl.waitUntilTime(startTime + ts * (k + 1));
        }
    }
}